/**
 * Copyright (c) 2008 SunGard
 *     All Rights Reserved
 *        CONFIDENTIAL
 */

package com.netcertainty.client.b2b;


import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringEscapeUtils;


/**
 * Utilities for submitting an SDD B2B request.
 */
public class SDDSubmit
{
    private String host;
    private String relativePortAddress;

    
    /**
     * Main method for stand-alone testing.
     * Submits an SDD file to the specified host. 
     * 
     * @param args  1. Host to submit to.
     *              2. Name of file to submit.
     */
    public static void main(String[] args)
    {
        // Validate the parameters.
        int pos = 0;
        boolean submit_request = true;
        boolean parse_options = true;
        String template_root_dir = ".";
        while (parse_options)
        {
            if (pos >= args.length) {showUsage(); return;}
            
            if ("-d".equals(args[pos]) || "/d".equals(args[pos]))
            {
                pos++;
                if (pos >= args.length) {showUsage(); return;}
                template_root_dir = args[pos];
            }
            else if ("-h".equals(args[pos]) || "/h".equals(args[pos]))
            {
                showUsage();
                return;
            }
            else if ("-showreq".equals(args[pos]) || "/showreq".equals(args[pos]))
            {
                submit_request = false;
            }
            else
            {
                parse_options = false;
                pos--;
            }
            
            pos++;
        }
        
        if (pos >= args.length) {showUsage(); return;}
        String host = args[pos];
        pos++;
        
        if (pos >= args.length) {showUsage(); return;}
        String req_file_name = template_root_dir + File.separator + args[pos];
        pos++;
        
        SDDSubmit sdd = new SDDSubmit(host, "/sdd/b2b/sdddc_ns.jsp");
        
        // Set up template parameter list
        int number_of_parameters = args.length - pos;
        List<String> transactionIds = new ArrayList<String>();
        Object[] parameters;
        if (number_of_parameters == 0)
        {
            parameters = new String[1];
            String id = sdd.generateTransactionId();
            transactionIds.add(id);
            parameters[0] = id;
        }
        else
        {
            parameters = new String[number_of_parameters];
            for (int i = pos, j = 0; i < args.length; i++, j++)
            {
                if ("-t".equals(args[i]) || "/t".equals(args[i]))
                {
                    String id = sdd.generateTransactionId();
                    transactionIds.add(id);
                    parameters[j] = id;
                }
                else if (args[i].startsWith("@@file64@@"))
                {
                	String file_name = args[i].substring(10);
                    try
					{
						parameters[j] = new String(Base64.encodeBase64(FileUtils.readFileToByteArray(new File(file_name))));
					}
					catch (IOException e)
					{
			            System.err.println("Error reading or base64 encoding file " + file_name + ": " + e.getMessage());
			            return;
					}
                }
                else if (args[i].startsWith("@@file@@"))
                {
                	String file_name = args[i].substring(8);
                    try
					{
						parameters[j] = FileUtils.readFileToString(new File(file_name));
					}
					catch (IOException e)
					{
			            System.err.println("Error reading file " + file_name + ": " + e.getMessage());
			            return;
					}
                }
                else if (args[i].startsWith("@@"))
                {
                    parameters[j] = args[i].substring(2);
                }
                else
                {
                    parameters[j] = StringEscapeUtils.escapeXml(args[i]);
                }
            }
        }
        
        
        // Set up a request with the indicated ID
        String requestXML;
        try
        {
            String requestXMLTemplate = FileUtils.readFileToString(new File(req_file_name));
            requestXML = String.format(requestXMLTemplate, parameters);
        }
        catch (Exception e)
        {
            System.err.println("Error constructing B2B request from " + req_file_name);
            e.printStackTrace();
            return;
        }
        
        // Gat the method ID
        String methodId;
        try
        {
            methodId = getMethodID(requestXML);
        }
        catch (Exception e1)
        {
            System.err.println("Unable to determine method ID for this request: " + req_file_name);
            return;
        }
        
        // Submit the request and get the response.
        String response = "";
        if (submit_request)
        {
            try
            {
                response = SDDHttpPost.doService(new URI(sdd.getPortAddress()), methodId, requestXML);
            }
            catch (Exception e)
            {
                System.err.println("Error submitting request to " + sdd.getPortAddress());
                e.printStackTrace();
                System.err.println("Request:");
                System.err.println(requestXML);
                return;
            }
        }
        
        // Show info
        System.err.println("Method: " + methodId);
        
        boolean first = true;
        Iterator<String> id = transactionIds.iterator();
        while (id.hasNext())
        {
            System.err.print((first) ? "Transaction ID: " : ", ");
            System.err.print(id.next());
            
            first = false;
        }
        System.err.println("");

        // Show result
        System.out.println((submit_request) ? response : requestXML);
    }
    
    private static void showUsage()
    {
        System.err.println("Usage: SDDSubmit [<options>] <Host> <RequestTemplateFileName> [<Parameters>]");
        System.err.println("  Options:");
        System.err.println("    -d <dir> - Set the template file root directory.");
        System.err.println("    -h - Print this help message.");
        System.err.println("    -showreq - Show the request after parameter substitution,");
        System.err.println("        but do not submit the request to the host.");
        System.err.println("  Parameters:");
        System.err.println("    Host - URL of host, no trailing \'/\'.");
        System.err.println("    RequestTemplateFileName - Path to request template file.");
        System.err.println("    Request Parameters - List of parameters for template.");
        System.err.println("        Parameters are automatically XML-encoded.");
        System.err.println("        Prefix parameter with \"@@\" to prevent XML encoding.");
        System.err.println("        Use \"@@file@@\" + file_name to load from a file.");
        System.err.println("        Use \"@@file64@@\" + file_name to load from a file and base64 encode.");
        System.err.println("        Use \"-t\" to automatically generate a transaction ID.");
        System.err.println("        If no parameters, a transaction ID will be generated as parameter 1");
        System.err.println("");
        System.err.println("  Template file should specify parameters as in this example:");
        System.err.println("      <TransactionID>%1$s</TransactionID>");
        System.err.println("  where the number is the parameter number, starting at 1.");
    }

    
    
    //----------- Constructors ---------------
    
    /**
     * @param host                 URL of host, no trailing '/'.
     * @param relativePortAddress  Relative URL of SDD B2B port. Must have leading '/'.
     */
    public SDDSubmit(String host, String relativePortAddress) {
        super();
        this.host = host;
        this.relativePortAddress = relativePortAddress;
    }
    

    //----------- Properties ---------------
    
    /**
     * @return the host
     */
    public String getHost() {
        return host;
    }


    /**
     * @param host the host to set
     */
    public void setHost(String host) {
        this.host = host;
    }


    /**
     * @return the relativePortAddress
     */
    public String getRelativePortAddress() {
        return relativePortAddress;
    }


    /**
     * @param relativePortAddress the relativePortAddress to set
     */
    public void setRelativePortAddress(String relativePortAddress) {
        this.relativePortAddress = relativePortAddress;
    }

    
    public String getPortAddress() {return (host + relativePortAddress);}
    
    

    //----------- Methods ---------------


    
    public String generateTransactionId()
    {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ssZ");
        Date date = new Date();
        return "Test-" + df.format(date) + "-" +
            Double.toString(Math.random() * 1000000).substring(0, 6);
    }

    /**
     * Get the method ID for the given request.
     * 
     * @param requestXML  The request.
     * @return            The method ID for the request.
     * @throws Exception  If the request is malformed such that no method ID can be inferred.
     */
    public static String getMethodID(String requestXML)
    throws Exception
    {
        // The method ID is the name of the first element, with the trailing "Rq" dropped.
        int pos1 = -1;
        char next;
        do // Skip XML directives
        {
            pos1 = requestXML.indexOf('<', ++pos1);
            if (pos1 == -1) throw new Exception("No method ID can be determined for the request.");
            next = requestXML.charAt(pos1 + 1);
        } while (next < 'A');
        int pos2 = requestXML.indexOf(' ', pos1);
        int pos3 = requestXML.indexOf('>', pos1);
        if (pos2 == -1 || pos3 < pos2) pos2 = pos3;
        if (pos2 == -1) throw new Exception("Malformed XML");
        return requestXML.substring(pos1 + 1, pos2 - 2);
    }
}
